# Generated by Django 2.2.4 on 2020-06-08 13:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='category',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='writing.Category', verbose_name='category'),
        ),
        migrations.AlterField(
            model_name='article',
            name='excerpt',
            field=models.TextField(help_text='preview', verbose_name='excerpt'),
        ),
        migrations.AlterField(
            model_name='article',
            name='seen',
            field=models.PositiveSmallIntegerField(db_index=True, default=0, verbose_name='seen'),
        ),
    ]



from django.db.models import Q
from django.views.generic import DetailView, ListView

from writing.models import Article, Category


class ArticleDetailView(DetailView):
    model = Article

    def dispatch(self, request, *args, **kwargs):
        article = self.get_object()

        # TODO: check cookie
        # update "seen"
        article.seen += 1
        article.save(update_fields=['seen'])

        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return super().get_queryset().select_related('author')


class ArticleListView(ListView):
    model = Article
    paginate_by = 10
    # filter_class = ArticleFilter

    def dispatch(self, request, *args, **kwargs):
        # self.filter = self.filter_class(request.GET, queryset=self.get_whole_queryset())
        return super().dispatch(request, *args, **kwargs)

    def get_whole_queryset(self):
        return self.model.objects.published() \
            .select_related('author', 'category') \
            .order_by('-created')

    def get_queryset(self):
        # return self.sort_queryset(self.filter.qs)
        return self.get_whole_queryset().filter(
            (Q(language=self.request.LANGUAGE_CODE) | Q(language__exact='') | Q(language__isnull=True))
        )

    def get_context_data(self, *args, **kwargs):
        context_data = super().get_context_data(*args, **kwargs)
        context_data['categories'] = Category.objects.filter(parent=None)
        # context_data['filter'] = self.filter
        return context_data



from django.conf import settings
from django.db import migrations, models
from martor.templatetags.martortags import safe_markdown
from writing.models import Article


def markdown_to_html(apps, schema_editor):
    Article = apps.get_model('writing', 'Article')

    articles = Article.objects.all()

    for article in articles:
        _migrate_content(article)

    Article.objects.bulk_update(articles, fields=['content'])


def _migrate_content(article):
    content_martor = article.content
    content_html = safe_markdown(content_martor)
    article.content = content_html


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0006_article_publish_date'),
    ]

    operations = [
        migrations.RunPython(markdown_to_html),
    ]



# Generated by Django 2.2.4 on 2020-06-08 13:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import martor.models
import mptt.fields
import writing.mixins


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, unique=True, verbose_name='title')),
                ('slug', models.SlugField(blank=True, max_length=150, unique=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='writing.Category', verbose_name='parent')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
                'ordering': ('title',),
                'default_permissions': ('list', 'view', 'add', 'change', 'delete'),
            },
            bases=(writing.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(db_index=True, max_length=100, unique=True, verbose_name='title')),
                ('slug', models.SlugField(blank=True, max_length=150, unique=True)),
                ('is_highlighted', models.BooleanField(default=False, verbose_name='highlighted')),
                ('is_published', models.BooleanField(default=True, verbose_name='published')),
                ('seen', models.PositiveSmallIntegerField(db_index=True, default=1, verbose_name='seen')),
                ('excerpt', martor.models.MartorField(verbose_name='excerpt')),
                ('content', martor.models.MartorField(verbose_name='content')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='author')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='writing.Category', verbose_name='category')),
            ],
            options={
                'verbose_name': 'article',
                'verbose_name_plural': 'articles',
                'ordering': ['category', 'title'],
                'default_permissions': ('list', 'view', 'add', 'change', 'delete'),
            },
        ),
    ]



from django.urls import path
from django.utils.translation import pgettext_lazy
from writing.views import ArticleListView, ArticleDetailView#, CategoryDetailView

app_name = 'writing'

urlpatterns = [
    # path('category/<slug:slug>/', CategoryDetailView.as_view(), name='category_detail'),
    # path(pgettext_lazy('url', 'articles/'), ArticleListView.as_view(), name='article_list'),
    path('', ArticleListView.as_view(), name='article_list'),
    path('<slug:slug>/', ArticleDetailView.as_view(), name='article_detail'),
]



# Generated by Django 5.0.9 on 2024-10-04 20:03

import tinymce.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0007_markdown_to_html'),
    ]

    operations = [
        migrations.AlterField(
            model_name='article',
            name='content',
            field=tinymce.models.HTMLField(verbose_name='content'),
        ),
    ]



from django.db.models import QuerySet, Q


class ArticleQuerySet(QuerySet):
    def highlighted(self):
        return self.filter(is_highlighted=True)

    def published(self):
        return self.filter(is_published=True)

    def of_category(self, category):
        return self.filter(
            Q(category=category) |
            Q(category__parent=category)
        )



from django import forms
from writing import settings as writing_settings
from django.contrib import admin
from mptt.admin import DraggableMPTTAdmin, TreeRelatedFieldListFilter

from writing.models import Category, Article

admin.site.register(Category, DraggableMPTTAdmin,
    search_fields=('title',),
    list_display=('tree_actions', 'indented_title', 'slug', 'created'),
    list_display_links=('indented_title',),
)


class ArticleAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['language'].widget = forms.Select(choices=writing_settings.LANGUAGES)


@admin.register(Article)
class ArticleAdmin(admin.ModelAdmin):
    search_fields = ('title', 'category__title', 'excerpt', 'content')
    date_hierarchy = 'created'
    list_display = ('title', 'category', 'language', 'author', 'is_highlighted', 'is_published', 'publish_date', 'created', 'modified')
    list_filter = ('is_highlighted', 'is_published', 'language', ('category', TreeRelatedFieldListFilter))
    list_editable = ('is_highlighted', 'is_published')
    list_select_related = ['category', 'author']
    raw_id_fields = ['author']
    # autocomplete_fields = ['category', 'author']

    form = ArticleAdminForm



# Generated by Django 3.1.14 on 2022-03-16 15:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0004_article_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='language',
            field=models.CharField(blank=True, db_index=True, max_length=2, verbose_name='language'),
        ),
    ]



# Generated by Django 3.1.14 on 2022-04-06 13:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0005_article_language'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='publish_date',
            field=models.DateField(blank=True, default=None, null=True, verbose_name='publish_date'),
        ),
    ]



# Generated by Django 2.2.4 on 2020-07-07 09:34

from django.db import migrations
import sorl.thumbnail.fields


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0003_auto_20200608_1554'),
    ]

    operations = [
        migrations.AddField(
            model_name='article',
            name='image',
            field=sorl.thumbnail.fields.ImageField(blank=True, help_text='photo or image', max_length=1024, upload_to='images', verbose_name='image'),
        ),
    ]



# Generated by Django 2.2.4 on 2020-06-08 13:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('writing', '0002_auto_20200608_1552'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='article',
            options={'default_permissions': ('list', 'view', 'add', 'change', 'delete'), 'ordering': ['title'], 'verbose_name': 'article', 'verbose_name_plural': 'articles'},
        ),
    ]



from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.text import slugify
from mptt.models import MPTTModel, TreeForeignKey
from sorl import thumbnail
from tinymce.models import HTMLField

from writing.mixins import SlugMixin
from writing.querysets import ArticleQuerySet

from django.utils.translation import gettext_lazy as _


class Category(SlugMixin, MPTTModel):
    FORCE_SLUG_REGENERATION = True

    parent = TreeForeignKey('self', verbose_name=_('parent'), related_name='children', on_delete=models.CASCADE,
        db_index=True, blank=True, null=True)
    title = models.CharField(_('title'), max_length=100, unique=True)
    slug = models.SlugField(unique=True, max_length=150, blank=True, db_index=True)
    # icon = models.CharField(_(u'icon'), max_length=255, blank=True)
    created = models.DateTimeField(_(u'created'), auto_now_add=True)
    modified = models.DateTimeField(_(u'modified'), auto_now=True)

    class Meta:
        verbose_name = _(u'category')
        verbose_name_plural = _(u'categories')
        ordering = ('title',)
        # order_insertion_by = ('title',)
        default_permissions = settings.DEFAULT_PERMISSIONS

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('writing:category_detail', kwargs={
            'slug': slugify(self.slug)
        })

    @cached_property
    def nested_article_set(self):
        descendants = self.get_descendants(include_self=True)
        return Article.objects \
            .filter(category__in=descendants) \
            .select_related('author', 'category')


class Article(SlugMixin, models.Model):
    FORCE_SLUG_REGENERATION = False  # To preserve SEO links

    category = models.ForeignKey(Category, verbose_name=_('category'), on_delete=models.PROTECT, blank=True, null=True, default=None)
    title = models.CharField(_('title'), max_length=100, unique=True, db_index=True)
    slug = models.SlugField(unique=True, max_length=150, blank=True, db_index=True)
    author = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('author'), on_delete=models.PROTECT)
    is_highlighted = models.BooleanField(_('highlighted'), default=False)
    is_published = models.BooleanField(_('published'), default=True)
    seen = models.PositiveSmallIntegerField(_('seen'), db_index=True, default=0)
    image = thumbnail.ImageField(
        verbose_name=_('image'),
        help_text=_('photo or image'),
        max_length=1024,
        upload_to='images',
        blank=True
    )
    excerpt = models.TextField(_('excerpt'), help_text=_('preview'))
    content = HTMLField(_('content'))
    language = models.CharField(_('language'), max_length=2, db_index=True, blank=True)
    publish_date = models.DateField(_(u'publish date'), blank=True, null=True, default=None)
    created = models.DateTimeField(_(u'created'), auto_now_add=True)
    modified = models.DateTimeField(_(u'modified'), auto_now=True)
    objects = ArticleQuerySet.as_manager()

    class Meta:
        verbose_name = _(u'article')
        verbose_name_plural = _(u'articles')
        ordering = ['title']
        default_permissions = settings.DEFAULT_PERMISSIONS

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('writing:article_detail', kwargs={
            'slug': slugify(self.slug)
        })



from django.core.validators import EMPTY_VALUES
from django.utils.text import slugify


class SlugMixin(object):  # TODO: move to django-pragmatic
    MAX_SLUG_LENGTH = 150
    FORCE_SLUG_REGENERATION = True
    SLUG_FIELD = 'title'

    def save(self, **kwargs):
        if self.slug in EMPTY_VALUES or self.FORCE_SLUG_REGENERATION:
            slug = slugify(getattr(self, self.SLUG_FIELD))
            self.slug = slug
            index = 1

            # Ensure uniqueness
            while self.__class__.objects.filter(slug=self.slug).exclude(pk=self.pk).exists():
                self.slug = f'{slug}-{index}'
                index += 1

        return super().save(**kwargs)
